<?php
/**
 * Plugin Name: SARE
 * Version: 1.0.1
 * Plugin URI: https://sare.pl
 * Description: SARE Wordpress plugin
 * Author: yasio
 * Author URI: https://yasio.dev
 * Requires at least: 5.6
 * Tested up to: 6.0.0
 * Requires PHP: 7.4
 *
 * Text Domain: sare
 * Domain Path: /languages/
 *
 * License: GPLv2 or later
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 */

/**
 * Copyright (c) 2018 Jan Baszczok on behalf of SARE (email: j.baszczok@sare.pl). All rights reserved.
 *
 * Released under the GPL license
 * http://www.opensource.org/licenses/gpl-license.php
 *
 * This is an add-on for WordPress
 * http://wordpress.org/
 *
 * **********************************************************************
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * **********************************************************************
 */

// don't call the file directly
if (!defined('ABSPATH')) {
    exit;
}

/**
 * Sare class
 *
 * @class Sare The class that holds the entire Sare plugin
 */
final class Sare
{

    /**
     * Plugin version
     *
     * @var string
     */
    public $version = '1.0.1';

    /**
     * Holds various class instances
     *
     * @var array
     */
    private $container = array();

    /**
     * Constructor for the Sare class
     *
     * Sets up all the appropriate hooks and actions
     * within our plugin.
     */
    public function __construct()
    {

        $this->define_constants();

        register_activation_hook(__FILE__, array($this, 'activate'));
        register_deactivation_hook(__FILE__, array($this, 'deactivate'));

        add_action('plugins_loaded', array($this, 'init_plugin'));
    }

    /**
     * Initializes the Sare() class
     *
     * Checks for an existing Sare() instance
     * and if it doesn't find one, creates it.
     */
    public static function init()
    {
        static $instance = false;

        if (!$instance) {
            $instance = new Sare();
        }

        return $instance;
    }

    /**
     * Magic getter to bypass referencing plugin.
     *
     * @param $prop
     *
     * @return mixed
     */
    public function __get($prop)
    {
        if (array_key_exists($prop, $this->container)) {
            return $this->container[$prop];
        }

        return $this->{$prop};
    }

    /**
     * Magic isset to bypass referencing plugin.
     *
     * @param $prop
     *
     * @return mixed
     */
    public function __isset($prop)
    {
        return isset($this->{$prop}) || isset($this->container[$prop]);
    }

    /**
     * Define the constants
     *
     * @return void
     */
    public function define_constants()
    {
        define('SARE_VERSION', $this->version);
        define('SARE_FILE', __FILE__);
        define('SARE_PATH', dirname(SARE_FILE));
        define('SARE_INCLUDES', SARE_PATH . '/includes');
        define('SARE_URL', plugins_url('', SARE_FILE));
        define('SARE_ASSETS', SARE_URL . '/assets');
        define('SARE_URI', 'https://s.enewsletter.pl/api/v1/');
    }

    /**
     * Load the plugin after all plugis are loaded
     *
     * @return void
     */
    public function init_plugin()
    {
        $this->includes();
        $this->init_hooks();
    }

    /**
     * Placeholder for activation function
     *
     * Nothing being called here yet.
     */
    public function activate()
    {

        $installed = get_option('sare_installed');

        if (!$installed) {
            update_option('sare_installed', time());
        }

        update_option('sare_version', SARE_VERSION);

        global $wpdb;
        $data = file_get_contents(SARE_PATH . '/includes/options.json');
        $data = json_decode($data, true);
        $data = serialize($data);

        $wpdb->query("SELECT * FROM $wpdb->options WHERE option_name = 'sare_data'");
        if ($wpdb->num_rows == 0) {
            $wpdb->query("INSERT INTO $wpdb->options (option_name, option_value) VALUES ('sare_data', '$data')");
        } else {
            $wpdb->query("UPDATE $wpdb->options SET option_value = '$data' WHERE option_name = 'sare_data'");
        }

        $wpdb->query("INSERT INTO $wpdb->options (option_name, option_value) VALUES ('sare_uid', '')");
        $wpdb->query("INSERT INTO $wpdb->options (option_name, option_value) VALUES ('sare_apikey', '')");
    }

    /**
     * Placeholder for deactivation function
     *
     * Nothing being called here yet.
     */
    public function deactivate()
    {
        // delete uid and apikey
        global $wpdb;
        $wpdb->query("DELETE FROM $wpdb->options WHERE option_name = 'sare_uid'");
        $wpdb->query("DELETE FROM $wpdb->options WHERE option_name = 'sare_apikey'");
    }

    /**
     * Include the required files
     *
     * @return void
     */
    public function includes()
    {

        require_once SARE_INCLUDES . '/class-assets.php';

        if ($this->is_request('admin')) {
            require_once SARE_INCLUDES . '/class-admin.php';
        }

        if ($this->is_request('frontend')) {
            require_once SARE_INCLUDES . '/class-frontend.php';
        }

        if ($this->is_request('ajax')) {
            // require_once SARE_INCLUDES . '/class-ajax.php';
        }

        if ($this->is_request('rest')) {
            require_once SARE_INCLUDES . '/class-rest-api.php';
        }
    }

    /**
     * Initialize the hooks
     *
     * @return void
     */
    public function init_hooks()
    {

        add_action('init', array($this, 'init_classes'));

        add_action('init', array($this, 'localization_setup'));

        add_action('wp_ajax_get_settings', [$this, 'get_settings']);
        add_action('wp_ajax_nopriv_get_settings', [$this, 'get_settings']);

        add_action('wp_ajax_save_settings', [$this, 'save_settings']);

        add_action('wp_ajax_get_uid_apikey', [$this, 'get_uid_apikey']);
        add_action('wp_ajax_nopriv_get_uid_apikey', [$this, 'get_uid_apikey']);

        add_action('wp_ajax_save_uid_apikey', [$this, 'save_uid_apikey']);

        add_action('wp_ajax_check_integration', [$this, 'check_integration']);
        add_action('wp_ajax_nopriv_check_integration', [$this, 'check_integration']);

        add_action('wp_ajax_get_sare_data', [$this, 'get_sare_data']);
        add_action('wp_ajax_nopriv_get_sare_data', [$this, 'get_sare_data']);

        add_action('wp_ajax_sare_submit', [$this, 'sare_submit']);
        add_action('wp_ajax_nopriv_sare_submit', [$this, 'sare_submit']);
    }

    /**
     * Instantiate the required classes
     *
     * @return void
     */
    public function init_classes()
    {

        if ($this->is_request('admin')) {
            $this->container['admin'] = new Sare\Admin();
        }

        if ($this->is_request('frontend')) {
            $this->container['frontend'] = new Sare\Frontend();
        }

        if ($this->is_request('ajax')) {
            // $this->container['ajax'] =  new Sare\Ajax();
        }

        if ($this->is_request('rest')) {
            $this->container['rest'] = new Sare\REST_API();
        }

        $this->container['assets'] = new Sare\Assets();
    }
    public function get_settings()
    {
        global $wpdb;
        $sare_data = $wpdb->get_var("SELECT option_value FROM $wpdb->options WHERE option_name = 'sare_data'");
        $sare_data = unserialize($sare_data);
        $sare_data = json_encode($sare_data);
        echo $sare_data;
        wp_die();
    }
    public function save_settings()
    {
        global $wpdb;
        $dataToSave = stripslashes_deep($_POST['settings']);
        $dataToSave = json_decode($dataToSave, true);
        $dataToSave = serialize($dataToSave);
        $wpdb->query("UPDATE $wpdb->options SET option_value = '$dataToSave' WHERE option_name = 'sare_data'");
        wp_die();
    }

    public function get_uid_apikey()
    {
        global $wpdb;
        $uid = $wpdb->get_var("SELECT option_value FROM $wpdb->options WHERE option_name = 'sare_uid'");
        $apikey = $wpdb->get_var("SELECT option_value FROM $wpdb->options WHERE option_name = 'sare_apikey'");
        $data = [
            'uid' => $uid,
            'apikey' => $apikey
        ];
        $data = json_encode($data);
        echo $data;
        wp_die();
    }

    public function save_uid_apikey()
    {
        global $wpdb;
        $uid = $_POST['uid'];
        $apikey = $_POST['apikey'];
        if (empty($uid) || !is_numeric($uid)) {
            header('HTTP/1.1 400 Bad Request');
            die('UID jest nieprawidłowy');
        }
        if (!preg_match('/^[0-9a-f]{8}-[0-9a-f]{4}-4[0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/', $apikey)) {
            header('HTTP/1.1 400 Bad Request');
            die('APIKEY jest nieprawidłowy');
        }

        $url = SARE_URI . $uid . '/status';
        $headers = [
            'ApiKey' => $apikey
        ];
        $response = wp_remote_get($url, [
            'headers' => $headers
        ]);
        if (is_wp_error($response)) {
            header('HTTP/1.1 400 Bad Request');
            die('Błąd połączenia z SARE API');
        }
        $response = json_decode($response['body'], true);
        
        if ($response['code'] != 200) {
            header('HTTP/1.1 400 Bad Request');
            die('Błąd połączenia z SARE API');
        }

        $wpdb->query("UPDATE $wpdb->options SET option_value = '$uid' WHERE option_name = 'sare_uid'");
        $wpdb->query("UPDATE $wpdb->options SET option_value = '$apikey' WHERE option_name = 'sare_apikey'");
        wp_die();
    }
    public function check_integration()
    {
        global $wpdb;
        $uid = $wpdb->get_var("SELECT option_value FROM $wpdb->options WHERE option_name = 'sare_uid'");
        if (empty($uid)) {
            echo 'not_ok';
        }
        $apikey = $wpdb->get_var("SELECT option_value FROM $wpdb->options WHERE option_name = 'sare_apikey'");
        if (empty($apikey)) {
            echo 'not_ok';
        }
        $url = SARE_URI . $uid . '/status';
        $headers = [
            'ApiKey' => $apikey
        ];
        $response = wp_remote_get($url, [
            'headers' => $headers
        ]);
        if (is_wp_error($response)) {
            header('HTTP/1.1 400 Bad Request');
            die('Niezintegrowany');
        }
        $response = json_decode($response['body'], true);
        
        if ($response['code'] != 200) {
            echo 'not_ok';
        }
        echo 'ok';
        wp_die();
    }

    public function get_sare_data()
    {
        global $wpdb;
        $uid = $wpdb->get_var("SELECT option_value FROM $wpdb->options WHERE option_name = 'sare_uid'");
        $apikey = $wpdb->get_var("SELECT option_value FROM $wpdb->options WHERE option_name = 'sare_apikey'");
        if (empty($uid) || empty($apikey)) {
            header('HTTP/1.1 400 Bad Request');
            die('Błąd połączenia z SARE API');
        }

        // get group list
        $url = SARE_URI . $uid . '/group/list';
        $headers = [
            'ApiKey' => $apikey
        ];
        $response = wp_remote_get($url, [
            'headers' => $headers
        ]);
        if (is_wp_error($response)) {
            header('HTTP/1.1 400 Bad Request');
            die('Błąd połączenia z SARE API');
        }
        $response = json_decode($response['body'], true);
        
        if ($response['code'] != 200) {
            header('HTTP/1.1 400 Bad Request');
            die('Błąd połączenia z SARE API');
        }
        $data = $response['response'];

        // get props list
        $url = SARE_URI . $uid . '/email/props';
        $responseProps = wp_remote_get($url, [
            'headers' => $headers
        ]);
        if (is_wp_error($responseProps)) {
            header('HTTP/1.1 400 Bad Request');
            die('Błąd połączenia z SARE API');
        }
        $responseProps = json_decode($responseProps['body'], true);
        
        if ($responseProps['code'] != 200) {
            header('HTTP/1.1 400 Bad Request');
            die('Błąd połączenia z SARE API');
        }
        $dataProps = $responseProps['response'];

        $output = json_encode([
            'groups' => $data,
            'props' => $dataProps,
        ]);
        echo $output;
        wp_die();
    }

    public function sare_submit()
    {
        global $wpdb;
        $email = $_POST['email'];
        if (empty($email) || !filter_var($email, FILTER_VALIDATE_EMAIL)) {
            header('HTTP/1.1 400 Bad Request');
            die('Email jest nieprawidłowy');
        }
        $agreement = $_POST['checkbox'];
        if ($agreement != 'on') {
            header('HTTP/1.1 400 Bad Request');
            die('Nie zaakceptowano zgody');
        }
        $uid = $wpdb->get_var("SELECT option_value FROM $wpdb->options WHERE option_name = 'sare_uid'");
        $apikey = $wpdb->get_var("SELECT option_value FROM $wpdb->options WHERE option_name = 'sare_apikey'");
        if (empty($uid) || empty($apikey)) {
            header('HTTP/1.1 400 Bad Request');
            die('Błąd połączenia z SARE API');
        }
        $sare_data = $wpdb->get_var("SELECT option_value FROM $wpdb->options WHERE option_name = 'sare_data'");
        $sare_data = unserialize($sare_data);
        $sare_data = $sare_data['options'];
        $prop = [];
        $prop_cust = [];

        if ($sare_data['show_custom_fields'] === true) {
            foreach ($sare_data['custom_fields'] as $field) {
                if (preg_match('/prop\d/', $field['prop'])) {
                    if (empty($_POST[$field['prop']])) continue;
                    if ($field['type'] === 'checkbox' && $_POST[$field['prop']] === 'on') {
                        $prop[$field['prop']] = 'tak';
                        continue;
                    } else if ($field['type'] === 'checkbox' && $_POST[$field['prop']] !== 'on') {
                        $prop[$field['prop']] = 'nie';
                        continue;
                    }
                    $prop[$field['prop']] = $_POST[$field['prop']];
                } else {
                    if (empty($_POST[$field['prop']])) continue;
                    if ($field['type'] === 'checkbox' && $_POST[$field['prop']] === 'on') {
                        $prop_cust[$field['prop']] = 'tak';
                        continue;
                    } else if ($field['type'] === 'checkbox' && $_POST[$field['prop']] !== 'on') {
                        $prop_cust[$field['prop']] = 'nie';
                        continue;
                    }
                    $prop_cust[$field['prop']] = $_POST[$field['prop']];
                }
            }
        }
        $data = stripslashes_deep($_POST);
        $url = SARE_URI . $uid . '/email/add';
        $body = [
            "only_add_to_groups" => $sare_data['only_add_to_groups'],
            "update_status_on_duplicate" => $sare_data['update_status_on_duplicate'],
            "update_on_duplicate" => true,
            "send_confirmation" => $sare_data['send_confirmation'],
            "update_user_data_after_confirmation" => $sare_data['update_user_data_after_confirmation'],
            "emails" => [[
                "email" => $email,
                "status" => $sare_data['status'],
                "groups" => $sare_data['groups'],
                "interface" => intval($sare_data['interface']),
                "mail_type" => [
                    "text",
                    "HTML"
                ],
                "prop" => $prop,
                "prop_cust" => $prop_cust,
            ]]
        ];
        if ($sare_data['show_gsm_field']) {
            $body['emails'][0]['gsm'] = $data['gsm'];
        }

        // die(var_dump($body));

        $headers = [
            'ApiKey' => $apikey,
            'Content-Type' => 'application/json'
        ];
        $response = wp_remote_post($url, [
            'headers' => $headers,
            'body' => json_encode($body)
        ]);
        if (is_wp_error($response)) {
            header('HTTP/1.1 400 Bad Request');
            die('Błąd połączenia z SARE API');
        }
        $response = json_decode($response['body'], true);
        if ($response['code'] != 200) {
            header('HTTP/1.1 400 Bad Request');
            die($response['status']);
        }
        echo json_encode($response);
        wp_die();
    }

    /**
     * Initialize plugin for localization
     *
     * @uses load_plugin_sare()
     */
    public function localization_setup()
    {
        load_plugin_textdomain('sare', false, dirname(plugin_basename(__FILE__)) . '/languages/');
    }

    /**
     * What type of request is this?
     *
     * @param  string $type admin, ajax, cron or frontend.
     *
     * @return bool
     */
    private function is_request($type)
    {
        switch ($type) {
            case 'admin':
                return is_admin();

            case 'ajax':
                return defined('DOING_AJAX');

            case 'rest':
                return defined('REST_REQUEST');

            case 'cron':
                return defined('DOING_CRON');

            case 'frontend':
                return (!is_admin() || defined('DOING_AJAX')) && !defined('DOING_CRON');
        }
    }

} // Sare

$sare = Sare::init();
